import numpy as np
import matplotlib.pyplot as plt
from hazma import spectra
from hazma.parameters import GF
from hazma.parameters import muon_mass as MMU

def msqrd(s, t):
   return 16.0 * GF**2 * t * (MMU**2 - t)

es = np.geomspace(1e-4, 1.0, 100) * MMU
final_states = ["e", "ve", "vm"]

dndes = {
   1: {
      # Photon
      r"$\gamma$ approx": spectra.dnde_photon(es, MMU, final_states, msqrd=msqrd),
      r"$\gamma$ analytic": spectra.dnde_photon_muon(es, MMU),
   },
   2: {
      # Positron
      r"$e^{+}$ approx": spectra.dnde_positron(es, MMU, final_states, msqrd=msqrd),
      r"$e^{+}$ analytic": spectra.dnde_positron_muon(es, MMU),
   },
   3: {
      # Electron and muon neutrino
      r"$\nu_e$ approx": spectra.dnde_neutrino(es, MMU, final_states, msqrd=msqrd, flavor="e"),
      r"$\nu_e$ analytic": spectra.dnde_neutrino_muon(es, MMU, flavor="e"),
      r"$\nu_{\mu}$ approx": spectra.dnde_neutrino(es, MMU, final_states, msqrd=msqrd, flavor="mu"),
      r"$\nu_{\mu}$ analytic": spectra.dnde_neutrino_muon(es, MMU, flavor="mu"),
   },
}


plt.figure(dpi=150, figsize=(12, 4))
for i, d in dndes.items():
   ax = plt.subplot(1, 3, i)
   lss = ["-", "--", "-", "--"]
   for j, (key, dnde) in enumerate(d.items()):
      ax.plot(es, dnde, ls=lss[j], label=key)
   ax.set_yscale("log")
   ax.set_xscale("log")
   ax.set_xlim(1e-1, 1e2)
   ax.set_ylim(1e-5, 1)
   if i == 1:
      ax.set_ylabel(r"$\dv{N}{E} \ [\mathrm{MeV}]$", fontdict=dict(size=16))
   ax.set_xlabel(r"$E \ [\mathrm{MeV}]$", fontdict=dict(size=16))
   ax.legend()

plt.tight_layout()